//
//  bank.cpp
//  
//
//  Created by Melanie on 11/11/14.
//
//

#include <stdio.h>
#include "bank.h"
#include "ff_globals.h"




Bank::Bank(int size, int melodyLength, bankKey key) {
    
    // create theme and variation objects
    Theme * theme = new Theme(melodyLength, 0, key);
    
    //text for each edit mode screen
    
    theme->text.set("Welcome to Fantastic Fantasy!\n\n Let's begin composing the theme!\n Click on the cubes to add notes, in a \n musical or visual shape that appeals to you.");
    melodic_input.push_back(theme);
    for (int i=1; i<size; i++) {
        Variation * var = new Variation(melodyLength, i, key, melodic_input.back());
        melodic_input.push_back(var);
        switch (i) {
            case 1: {
                var->text.set ("Congratulations! You have unlocked Variation 1.\n\n For this variation, consider adding scalar\n patterns, or diagonal lines of notes.");
                break;
            }
            case 2: {
                var->text.set ("Congratulations! You have unlocked Variation 2.\n\n For this variation, consider adding thirds, or stacking\n cubes vertically with one empty cube in between.");
                break;
            }
            case 3: {
                var->text.set ("Congratulations! You have unlocked Variation 3.\n\n For this variation, consider adding a tonic pedal, or\n repeated iterations of the final computer generated note \n underneath other musical material.");
                break;
            }
            case 4: {
                var->text.set ("Congratulations! You have unlocked Variation 4.\n\n For this variation, consider embellishing the portion after\n the final computer generated notes. You might add thirds, or \n octaves on top of the last computer generated note; an octave \n is the note 8 cubes away in the vertical direction.");
                break;
            }
            case 5: {
                var->text.set ("Congratulations! You have unlocked Variation 5.\n\n Consider the tools you've learned and have fun!");
                break;
            }
            case 6: {
                var->text.set ("Congratulations! You have unlocked Variation 6.\n\n Consider the tools you've learned and have fun!");
                break;
            }
            case 7: {
                var->text.set ("Congratulations! You have unlocked Variation 7.\n\n Consider the tools you've learned and have fun!");
                break;
            }



        }
    }
    
    active_melody = 0;
    active_track = 0;
    active_tracks = 8;
    
}

Bank::~Bank() {
    
}

//get active melody
Melody* Bank::getCurrentMelody(){
    return melodic_input[active_melody];
}

void Bank::playBank() {
    for (int i = 0; i< melodic_input.size(); i++) {
        //loop through buffer and play--see line 98 of bk audio.cpp
    }
}


void Bank::render() {
    
    if (Globals::mode == FF_EDIT) {
          glPushMatrix();
        // in edit mode, render only active melody
        melodic_input[active_melody]->renderOne();
          glPopMatrix();
    }
    else if (Globals::mode == FF_PLAYBACK){
        glPushMatrix();
 
        glTranslatef(-0.8, .4, 0);
        
        
        // render every melody in listening mode
        for (int i=0; i<melodic_input.size(); i++) {
            
            glColor3f(.5 * 1,.5 * .07,.4 * i - i * .257);
            glPushMatrix();
            if (i==active_melody && Globals::mainclock.isPlaying) {
                glScalef(.8, .8, .8);
                glTranslatef (0, -.08 * (Globals::mainclock.whichBeat()+1), 0);
            
                //makes them blink to the beat
                if (Globals::mainclock.whichBeat() % 4 == 0)
                    {
                     
                         glColor3f(.4, .6, 1);
                    }
            }
            glTranslatef(0.1, 0, 0);
            melodic_input[i]->renderAll();
            
            glPopMatrix();
            
            glPushMatrix();
            glTranslatef(0, 0.025, 0);
            float yStretch = 2.6;
            
            //operates the teapot animations
            if (i==active_melody && Globals::mainclock.isPlaying) {
                float f = (float)Globals::windowWidth / Globals::windowHeight;
                glScalef(1, f, 1);
                glRotatef(-45, 0, 0, 1);
                glScalef(1, yStretch/f, 1);
            } else {
                glScalef(1, yStretch, 1);
            }
            if (Globals::mainclock.whichBeat() % 2 == 0)
            {
                
                
                glTranslatef(.002,.002,0);
                glRotatef(13, 1,1,0);
            
            }
                glutSolidTeapot(0.05);
            
            

            glPopMatrix();

            glTranslatef(+1.6/(melodic_input.size()-1), 0, 0);
        }
        glPopMatrix();
    }
    
    
}
//no longer used except to rotate appropriate notes that are off limits for a particular key or melody
void Bank::selectTrack(int track_id) {
    active_track = track_id;
    
}
//toggles notes
void Bank::toggleNote(int step) {
    melodic_input[active_melody]->toggleNote(active_track, step);
}

void Bank::newStep(int step) {
    melodic_input[active_melody]->newStep(step);
    
    if (Globals::mode == FF_PLAYBACK) {
        if (Globals::mainclock.whichBeat() == (Globals::mainclock.stepsInGrid-1)) {
            Globals::bank.active_melody = ++Globals::bank.active_melody % Globals::bank.melodic_input.size();            
        }
    }
}
