//-----------------------------------------------------------------------------
// name: ff_audio.cpp
// desc: audio stuff for bokeh
//
// author: Ge Wang (ge@ccrma.stanford.edu)
//   date: 2013
//-----------------------------------------------------------------------------
#include "ff_audio.h"
#include "ff_globals.h"
#include "ff_sim.h"
#include "ff_gfx.h"
#include "y-fft.h"
#include "y-waveform.h"
//#include "ff_me.h"
#include <iostream>
using namespace std;




//-----------------------------------------------------------------------------
// name: audio_callback
// desc: audio callback
//-----------------------------------------------------------------------------
static void audio_callback( SAMPLE * buffer, unsigned int numFrames, void * userData )
{
    // sum
    SAMPLE sum = 0;
    // num channels
    unsigned int channels = Globals::lastAudioBufferChannels;
    
    // zero out
    memset( Globals::lastAudioBuffer, 0,
           sizeof(SAMPLE)*Globals::lastAudioBufferFrames*channels );
    memset( Globals::lastAudioBufferMono, 0,
           sizeof(SAMPLE)*Globals::lastAudioBufferFrames );
    
    // copy to global buffer
    memcpy( Globals::lastAudioBuffer, buffer,
           sizeof(SAMPLE)*numFrames*channels );
 
    
    // loop through buffer
    for( int i = 0; i < numFrames; i++ )
    {
    
        
        if (Globals::mainclock.checkBeat()) {
            Globals::bank.newStep(Globals::mainclock.whichBeat());
            
            // play a tick every beat
            if (Globals::mainclock.whichBeat() % 4 == 0) {//add globals settign
                if (Globals::mode==FF_EDIT && Globals::modes==BEAT) {
                Globals::synth->programChange( 0, 13);//change right side
                Globals::synth->noteOn( 0, 72, 60 );
                    }

            }
        }
        
    }
    
    Globals::synth->synthesize2( buffer, numFrames );

}




//-----------------------------------------------------------------------------
// name: ff_audio_init()
// desc: initialize audio system
//-----------------------------------------------------------------------------
bool ff_audio_init( unsigned int srate, unsigned int frameSize, unsigned channels, int input_device = 0, int output_device = 0 )
{
    // initialize
    if( !XAudioIO::init( input_device, output_device, srate, frameSize, channels, audio_callback, NULL ) )
    {
        // done
        return false;
    }
    
    // allocate
    Globals::lastAudioBuffer = new SAMPLE[frameSize*channels];
    // allocate mono buffer
    Globals::lastAudioBufferMono = new SAMPLE[frameSize];
    // allocate window buffer
    Globals::audioBufferWindow = new SAMPLE[frameSize];
    // set frame size (could have changed in XAudioIO::init())
    Globals::lastAudioBufferFrames = frameSize;
    // set num channels
    Globals::lastAudioBufferChannels = channels;
    
    // compute the window
    hanning( Globals::audioBufferWindow, frameSize );
    
    // create waveform
    Globals::waveform = new YWaveform();
    // place it
    Globals::waveform->loc.y = 1.5f;
    // set the width
    Globals::waveform->setWidth( 2.5f );
    // set the height
    Globals::waveform->setHeight( .75f );
    // initialize it
    Globals::waveform->init( frameSize );
    // active?
    Globals::waveform->active = Globals::renderWaveform;
    
    // add to sim
    Globals::sim->root().addChild( Globals::waveform );
    
    return true;
}




//-----------------------------------------------------------------------------
// name: vq_audio_start()
// desc: start audio system
//-----------------------------------------------------------------------------
bool ff_audio_start()
{
    // start the audio
    if( !XAudioIO::start() )
    {
        // done
        return false;
    }
    
    return true;
}
