//-----------------------------------------------------------------------------
// name: ff_globals.h
// desc: global stuff for bokeh visualization
//
// author: Ge Wang (ge@ccrma.stanford.edu)
//   date: 2013
//-----------------------------------------------------------------------------
#ifndef __FF_GLOBALS_H__
#define __FF_GLOBALS_H__


#include "x-def.h"
#include "x-audio.h"
#include "x-gfx.h"
#include "x-vector3d.h"
#include "y-waveform.h"
#include "y-fluidsynth.h"
#include "clock.h"
#include "bank.h"

// c++
#include <string>
#include <map>
#include <vector>
#include <utility>


// defines
#define FF_SRATE        44100
#define FF_FRAMESIZE    512
#define FF_NUMCHANNELS  2
#define FF_MAX_TEXTURES 32

//set up size and tracks of sequencer
#define STEPS_IN_MELODY 32
#define NUM_VARIATIONS 7




// mouse picking buffer
#define PICK_BUFSIZE 512

//-----------------------------------------------------------------------------
// name: enum BokehTextureNames
// desc: texture names
//-----------------------------------------------------------------------------
enum BokehTextureNames
{
    ff_TEX_NONE = 0,
    ff_TEX_FLARE_TNG_1,
    ff_TEX_FLARE_TNG_2,
    ff_TEX_FLARE_TNG_3,
    ff_TEX_FLARE_TNG_4,
    ff_TEX_FLARE_TNG_5
};



enum FFMode {
    FF_EDIT,
    FF_PLAYBACK,
    BEAT,
    NOBEAT
};


// forward reference
class BKSim;




//-----------------------------------------------------------------------------
// name: class Globals
// desc: the global class
//-----------------------------------------------------------------------------
class Globals
{
public:
    // top level root simulation
    static BKSim * sim;
    
    // path
    static std::string path;
    // path to datapath
    static std::string relpath;
    // datapath
    static std::string datapath;
    // version
    static std::string version;

    // last audio buffer
    static SAMPLE * lastAudioBuffer;
    static SAMPLE * lastAudioBufferMono;
    static SAMPLE * audioBufferWindow;
    static unsigned int lastAudioBufferFrames;
    static unsigned int lastAudioBufferChannels;

    // waveform
    static YWaveform * waveform;

    // width and height of the window
    static GLsizei windowWidth;
    static GLsizei windowHeight;
    static GLsizei lastWindowWidth;
    static GLsizei lastWindowHeight;

    // graphics fullscreen
    static GLboolean fullscreen;
    // render waveform
    static GLboolean renderWaveform;
    // blend pane instead of clearing screen
    static GLboolean blendScreen;
    // blend screen parameters
    static Vector3D blendAlpha;
    static GLfloat blendRed;
    // fill mode
    static GLenum fillmode;
    // background color
    static iSlew3D bgColor;
    // view stuff
    static Vector3D viewRadius;
    static Vector3D viewEyeY;
    static Vector3D fov;
    
    // textures
    static GLuint textures[];
    
    // light 0 position
    static GLfloat light0_pos[4];
    // light 1 parameters
    static GLfloat light1_ambient[4];
    static GLfloat light1_diffuse[4];
    static GLfloat light1_specular[4];
    static GLfloat light1_pos[4];
    // line width
    static GLfloat linewidth;
    // do
    static GLboolean fog;
    static GLuint fog_mode[4];   // storage for three/four types of fog
    static GLuint fog_filter;    // which fog to use
    static GLfloat fog_density;  // fog density
    
    // colors
    static Vector3D ourWhite;
    static Vector3D ourRed;
    static Vector3D ourBlue;
    static Vector3D ourOrange;
    static Vector3D ourGreen;
    static Vector3D ourGray;
    static Vector3D ourYellow;
    static Vector3D ourSoftYellow;
    static Vector3D ourPurple;
    
    // clock for sequencer
    static Clock mainclock;
    static Bank bank;
    static FFMode mode;
    static YFluidSynth * synth;
    static FFMode modes;
    
    // offscreen rendering buffer for picking
    static GLuint selectBuf[PICK_BUFSIZE];
};





#endif
