//
//  melody.cpp
//
//
//  Created by Melanie on 11/11/14.
//
//

#include "melody.h"
#include "ff_globals.h"
#include "bank.h"
#include "track.h"
#include <iostream>


//droplet constructor

Droplet::Droplet() {

    float radius = 0.1;

    Vector3D size(radius, radius, radius);
    Vector3D height(5, 1, 1);

    sphere.size = size;
    sphere.stacks = 60;
    sphere.slices = 60;

    cone.size = size;
    cone.height = height;
    cone.slices = 60;
    cone.stacks = 60;

    addChild(&cone);
    addChild(&sphere);
}

void Droplet::render() {

    glPushMatrix();

    glScalef (.1, .1, .1);

    sphere.render();


    glRotatef(-90, 1, 0, 0);


    cone.render();
    

    glPopMatrix();
}

//melody constructor

Melody::Melody(int numSteps, int eyeD, bankKey keyd) {

    length = numSteps;
    melody_id = eyeD;
    key = keyd;
       for (int i=0; i<NUM_NOTES_IN_MELODY; i++) {
        melody.push_back(Track(length, i));
    }
   
    text.set("");
    text.setWidth(1.5);
    text.fade (0,0);
    text.setStretch(.85);
    
    texts.set("");
    texts.setWidth(1.5);
    texts.fade (0,0);
    texts.setStretch(.85);
    
    

  
}

//renders one edit mode view, the unplayable notes are those that for a particular key or melody are unable to be played (user constraints)
void Melody::renderOne() {

    glPushMatrix();
   
    
    glTranslatef(-.95, -.75, 0); // overall displacement of all tracks
    for (int i=0; i<melody.size(); i++) {
        glTranslatef(0, .1, 0); // each track moves up a bit


        glPushMatrix();
        if (key==CMAJOR && Globals::bank.active_track == i) {
            glRotatef(30, 1, 0, 0);
        }
       
        if (key==CMAJOR) {
        for (int i=0; i< 32; i++) {
        melody[8].steps[i].is_unPlayable = true;
        }
        }
        if (key==AMINOR) {
            for (int i=0; i< 32; i++) {
                melody[7].steps[i].is_unPlayable = true;
            }
        }
        
        for (int i=0; i<12; i++) {
            melody[i].steps[0].is_unPlayable = true;
        }
        
        for (int i=0; i<12; i++) {
            melody[i].steps[23].is_unPlayable = true;
        }
        
        for (int i=0; i<12; i++) {
            melody[i].steps[24].is_unPlayable = true;
        }
        if (Globals::bank.getCurrentMelody()->melody_id== 0) {
            for (int i=0; i<12; i++){
            melody[i].steps[25].is_unPlayable = true;
            melody[i].steps[26].is_unPlayable = true;
            melody[i].steps[27].is_unPlayable = true;
            melody[i].steps[28].is_unPlayable = true;
            melody[i].steps[29].is_unPlayable = true;
            melody[i].steps[30].is_unPlayable = true;
            melody[i].steps[31].is_unPlayable = true;
            }
        }

        
        melody[i].renderDisplay();
     
      
        glPopMatrix();

    }

    // render associated text
    glPushMatrix();
    glTranslatef(0, 0.4, 0);
    float s = 0.5;
    glScalef(s, s, s);
    text.setCenterLocation(Vector3D(0, 0, 0));
    text.setColor(Vector3D (1,1,1));
    text.render();
    glPopMatrix();
    
    
    glPopMatrix();
}

void Melody::renderAll() { //renders listening mode

   
    glPushMatrix();
  
        // This is coupling things that shouldn't be coupled!
    if (melody_id == Globals::bank.active_melody)
    {
       
        for (int i=0; i<max(Globals::mainclock.whichBeat(), 0); i++) {
            glTranslatef(0,.06,0);
            glScalef(0.9, 0.9, 0.9);
            glColor3f(1, .1 * i, 0);
            droplet.render();
        }
    }
    
    glPopMatrix();
    
    glPushMatrix();
    
    //render the labels for the droplets in listening mode
    
    if (melody_id == Globals::bank.active_melody && Globals::mainclock.isPlaying){
        if (melody_id == 0) {
            texts.set("Theme!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 1) {
            texts.set("Variation1!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 2) {
            texts.set("Variation2!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 3) {
            texts.set("Variation3!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 4) {
            texts.set("Variation4!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 5) {
            texts.set("Variation5!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        if (melody_id == 6) {
            texts.set("Variation6!");
            texts.setWidth(.2);
            texts.setStretch(.5);
            glTranslatef (0,-.09,0);
            texts.render();
        }
        
        
        
    }
    glPopMatrix();
    

}

void Melody::newStep(int step) {
    for (int i=0; i<melody.size(); i++) {
        melody[i].newStep(step);
    }
}

//if melody has enough notes users can move on to the next one in edit mode
bool Melody::isComplete() {
    return (inputMelody.size() >= skeleton.size() * 0.5);
}

//toggle note if note is toggleable
void Melody::toggleNote(int note, int step) {

    if (key == CMAJOR && note == 0) return;
    if (key == CMAJOR && note == 8) return;
    if (key == AMINOR && note == 7) return;
    
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 25) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 26) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 27) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 28) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 29) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 30) return;
    if (Globals::bank.getCurrentMelody()->melody_id== 0 && step == 31) return;
    
    if (step == 0 || step == 23 || step == 24) return;


    bool is_step_on = melody[note].handleCubeInteraction(step);

    map<int, set<int> >::iterator it;

    if (is_step_on) { // add to melody map
        it = inputMelody.find(step);
        if (it != inputMelody.end()) {
            set<int> notes;
            inputMelody[step] = notes;
        }
        inputMelody[step].insert(note);
    }
    else {  // remove from melody map
        it = inputMelody.find(step);
        if (it == inputMelody.end()) {
            cerr << "trying to remove a note that doesn't exist (at least not in this step)" << endl;
        } else {
            inputMelody[step].erase(note);
            unLockStep (note, step);
        }
    }
}

//plays note if note can be played
void Melody::lockStep(int note, int stepNumber){
    

    
    if (key == CMAJOR && note == 0) return;
    if (key == CMAJOR && note == 8) return;
    if (key == AMINOR && note == 7) return;
    

    skeleton.insert(std::pair <int, int> (stepNumber, note));
    melody[note].steps[stepNumber].is_locked = true;
    melody[note].steps[stepNumber].is_on = true;
    
   
}

//takes note out of melody if it can be taken out or manipultaed
void Melody::unLockStep(int note, int stepNumber){


    if (key == CMAJOR && note == 0) return;
    if (key == CMAJOR && note == 8) return;
    if (key == AMINOR && note == 7) return;
  
        
    std::map<int, int>::iterator it = skeleton.find(stepNumber);

    if (it != skeleton.end()) {
       
        skeleton.erase(stepNumber);
        melody[note].steps[stepNumber].is_locked = false;
        melody[note].steps[stepNumber].is_on = false;
    }
}


//selected melody so can reorder twos o you can rearragnge

